from typing import Optional

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_core.pipes.date_time_offset_pipe import DateTimeOffsetPipe
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class UserGroup(DiscordCommandABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: CommandLogger,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            permissions: PermissionServiceABC,
            servers: ServerRepositoryABC,
            users: UserRepositoryABC,
            user_joined_servers: UserJoinedServerRepositoryABC,
            user_joined_voice_channel: UserJoinedVoiceChannelRepositoryABC,
            translate: TranslatePipe,
            date: DateTimeOffsetPipe
    ):
        DiscordCommandABC.__init__(self)

        self._config = config
        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._permissions = permissions
        self._servers = servers
        self._users = users
        self._user_joined_servers = user_joined_servers
        self._user_joined_voice_channel = user_joined_voice_channel
        self._t = translate
        self._date = date

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.hybrid_group()
    @commands.guild_only()
    async def user(self, ctx: Context):
        pass

    @user.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def info(self, ctx: Context, member: Optional[discord.Member] = None, *, wait: int = None):
        self._logger.debug(__name__, f'Received command user-info {ctx}:{member},{wait}')

        if member is None or not isinstance(member, discord.Member):
            member = ctx.author

        server = self._servers.find_server_by_discord_id(ctx.guild.id)
        user = self._users.find_user_by_discord_id_and_server_id(member.id, server.server_id)
        joins = self._user_joined_servers.get_user_joined_servers_by_user_id(user.user_id)

        embed = discord.Embed(
            title=member.name,
            description=member.name,
            color=int('ef9d0d', 16)
        )

        ujvs = self._user_joined_voice_channel.get_user_joined_voice_channels_by_user_id(user.user_id)
        ontime = ujvs.sum(lambda join: round((join.leaved_on - join.joined_on).total_seconds() / 3600, 2))

        embed.add_field(name=self._t.transform('modules.base.user_info.fields.id'), value=member.id)
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.name'), value=member.name)
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.discord_join'), value=self._date.transform(member.created_at), inline=False)
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.last_join'), value=self._date.transform(member.joined_at), inline=False)
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.xp'), value=str(user.xp))
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.ontime'), value=str(ontime))

        roles = ''
        for role in member.roles:
            roles += f'{role.name}\n'
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.roles'), value=roles, inline=False)

        joins_string = ''
        for join in joins:
            joins_string += f'{self._date.transform(join.joined_on)}\n'
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.joins'), value=joins_string)

        lefts_string = ''
        for join in joins:
            if join.leaved_on is None:
                if lefts_string == '':
                    lefts_string = '/'
                continue
            lefts_string += f'{self._date.transform(join.leaved_on)}\n'
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.lefts'), value=lefts_string)
        embed.add_field(name=self._t.transform('modules.base.user_info.fields.warnings'), value=self._t.transform('common.not_implemented_yet'), inline=False)

        await self._message_service.send_ctx_msg(ctx, embed, wait_before_delete=wait)
        self._logger.trace(__name__, f'Finished user-info command')
